@@ -9,11 +9,148 @@ from datetime                           import time
 
 from sqlalchemy.orm                     import class_mapper
 from sqlalchemy.exc                     import InvalidRequestError
+from sqlalchemy.orm.properties          import RelationshipProperty
 from sqlalchemy                         import not_
 from zope.interface                     import providedBy
 
 import colander
 import venusian
+import re
+import urllib
+
+def get_order_by(order_string):
+    final_orders = []
+    orders = [x.strip() for x in order_string.split(',')]
+
+    for order in orders:
+        if ' ' in order:
+            order, method = order.split()
+            if not method:
+                method = 'desc'
+            final_orders.append((order, method))
+
+    return final_orders
+
+def parse_key(key):
+    """ This will return a set of query rules based on the
+    key.
+
+
+    The structure of this is:
+    /column{column1.equals(foo).limit(0, 12).order_by(foo,bar)
+
+    for example:
+    /messages{id.not_equals(2), topic.equals(bar)}.order_by(name).limit(0, 10)
+    """
+
+    # first value will be our column
+    # rest will be values for rules on the query
+    if key.startswith("/"):
+        key = key[1:]
+
+    key_regex = re.compile("^(?P<name>\w+)(?P<columns>\{.+?\})?.?(?P<limit>limit\(.+?\))?.?(?P<order_by>order_by\(.+?\))?$")
+    match = re.match(key_regex, key)
+
+    name = match.group('name')
+    limit = match.group('limit')
+    order_by = match.group('order_by')
+    columns = match.group('columns')
+
+    result = {'table': name, 'column_filters': []}
+
+    if order_by:
+        order_regex = re.compile("^(\w+)\((.+?)\)$")
+        match = re.match(order_regex, order_by)
+        result['orders'] = []
+        orders = get_order_by(match.group(2))
+
+        for order, method in orders:
+            result['order_by'].append((order, method))
+
+    if limit:
+        limit_regex = re.compile("^(\w+)\((\d+),(\d+)\)$")
+        match = re.match(limit_regex, limit)
+        start = int(match.group(2))
+        end = int(match.group(3))
+        result['limit'] = (start, end)
+
+
+    if columns:
+        column_regex = re.compile("\w+\.\w+\(.+?\)")
+
+        results = re.findall(column_regex, columns)
+
+        for match in results:
+            filter_regex = re.compile("^(?P<column>\w+)\.(?P<command>\w+)\((?P<args>.+?)\)")
+            filter_match = re.match(filter_regex, match)
+
+            column = filter_match.group('column')
+            command = filter_match.group('command')
+            args = filter_match.group('args')
+
+            if command in ["equals", 'not_equals']:
+                # should be like equals(value)
+                # should be like not_equals(value)
+                result['column_filters'].append((column, command, args))
+            elif command in ["in", "not_in"]:
+                # should be like in(1,2,3)
+                # should be like not_in(1,2,3)
+                final_args = [x.strip() for x in args.split(',')]
+                result['column_filters'].append((column, command, final_args))
+
+    return result
+
+def filter_query(filters, query, cls):
+    for key, value in filters.iteritems():
+        if key == 'limit':
+            query = query.limit(value[1])
+            query = query.offset(value[0])
+        elif key == 'order_by':
+            for order, method in value:
+                prop = getattr(cls, order)
+
+                if method == 'desc':
+                    query = query.order_by(prop.desc())
+                else:
+                    query = query.order_by(prop.asc())
+        elif key == 'column_filters':
+            for column, command, args in value:
+                prop = getattr(cls, column)
+
+                if command == 'equals':
+                    query = query.filter(prop == args)
+                elif command == 'not_equals':
+                    query = query.filter(prop != args)
+                elif command == 'in':
+                    query = query.filter(prop.in_(args))
+                elif command == 'not_in':
+                    query = query.filter(not_(prop.in_(args)))
+
+    return query
+
+
+def filter_list(filters, list_):
+    for column, command, args in filters['column_filters']:
+        if command == 'equals':
+            list_ = filter(lambda x: getattr(x, column) == args, list_)
+        elif command == 'not_equals':
+            list_ = filter(lambda x: getattr(x, column) != args, list_)
+        elif command == 'in':
+            list_ = filter(lambda x: getattr(x, column) in args, list_)
+        elif command == 'not_in':
+            list_ = filter(lambda x: getattr(x, column) not in args, list_)
+
+    for key, value in filters.iteritems():
+        if key == 'limit':
+            list_ = list_[value[0]:value[1]]
+        elif key == 'order_by':
+            for order, method in value:
+                if method == 'asc':
+                    list_.sort(key=lambda x: getattr(x, order))
+                else:
+                    list_.sort(key=lambda x: getattr(x, order), reverse=True)
+
+    return list_
 
 def filter_list_by_qs(qs, collection):
     """ This function takes a querystring and a list
@@ -269,7 +406,20 @@ class JsonSerializableMixin(TraversalBase):
 
         # we make a copy because the dict will change if the database
         # is updated / flushed
-        options = self.__dict__.copy()
+    #    options = self.__dict__.copy()
+        properties = list(class_mapper(type(self)).iterate_properties)
+
+        relationships = [
+            p.key for p in properties if type(p) is RelationshipProperty
+        ]
+
+        attrs = []
+        all_properties = {}
+        for p in properties:
+            all_properties[p.key] = p
+
+            if not p.key in relationships:
+                attrs.append(p.key)
 
         # setup the blacklist
         # use set for easy 'in' lookups
@@ -278,15 +428,11 @@ class JsonSerializableMixin(TraversalBase):
         # extend the base blacklist with the json blacklist
         blacklist.update(getattr(self, '_json_blacklist', []))
 
-        for key in options:
+        for key in attrs:
             # skip blacklisted properties
             if key in blacklist:
                 continue
 
-            # do not include private and SQLAlchemy properties
-            if key.startswith(('__', '_sa_')):
-                continue
-
             # format and date/datetime/time properties to isoformat
             obj = getattr(self, key)
 
@@ -297,33 +443,39 @@ class JsonSerializableMixin(TraversalBase):
             # get the class property value
             attr = getattr(self, key)
 
-            # let see if we need to eagerly load it
-            # this is for SQLAlchemy foreign key fields that
-            # indicate with one-to-many relationships
-            if key in json_eager_load and attr:
-                if hasattr(attr, '_sa_instance_state'):
-                    props[key] = self.try_to_json(request, attr)
-                else:
-                    # jsonify all child objects
-                    props[key] = [self.try_to_json(request, x) for x in attr]
-                continue
-
-            # convert all non integer strings to string or if string conversion
-            # is not possible, convert it to Unicode
+            # convert all non integer strings to unicode or if unicode conversion
+            # is not possible, convert it to a byte string.
             if attr and not isinstance(attr, (int, float)):
                 try:
-                    props[key] = str(attr)
-                except UnicodeEncodeError:
-                    props[key] = unicode(attr)  # .encode('utf-8')
+                    props[key] = unicode(attr)
+                except UnicodeDecodeError:
+                    props[key] = str(attr)  # .encode('utf-8')
                 continue
 
             props[key] = attr
 
+        for key in relationships:
+            # get the class property value
+            attr = getattr(self, key)
+
+            # let see if we need to eagerly load it
+            # this is for SQLAlchemy foreign key fields that
+            # indicate with one-to-many relationships
+            many_directions = ["ONETOMANY", "MANYTOMANY"]
+
+            if key in json_eager_load and attr:
+                if all_properties[key].direction.name in many_directions:
+                    # jsonify all child objects
+                    props[key] = [self.try_to_json(request, x) for x in attr]
+                else:
+                    props[key] = self.try_to_json(request, attr)
+
         return props
 
 class ModelCollection(TraversalBase):
-    def __init__(self, collection):
+    def __init__(self, collection, request=None):
         self.collection = collection
+        self._request = request
 
     def __getitem__(self, key):
         """
@@ -409,21 +561,26 @@ class TraversalMixin(JsonSerializableMixin):
         """
         obj = None
 
+        filters = parse_key(attribute)
+
+        table_name = filters['table']
+
         # _request comes from our traversal code
         if hasattr(self, '_request'):
             #TODO: Lets make this less of a hack
             # POST means "create", so we are always looking for a class
-            if self._request.method == 'POST' and \
-                    self._request.path.endswith(attribute):
+            path = urllib.unquote(self._request.path)
 
-                cls = get_prop_from_cls(self.__class__, attribute)
+            if self._request.method == 'POST' and path.endswith(attribute):
+
+                cls = get_prop_from_cls(self.__class__, table_name)
 
                 cls.__parent__ = self
                 cls._request = self._request
 
                 return cls()
 
-        obj = getattr(self, attribute)
+        obj = getattr(self, table_name)
 
         # The model had the specific attribute, so we just need to figure out
         # if we are returning a collection or a single instance
@@ -441,7 +598,7 @@ class TraversalMixin(JsonSerializableMixin):
                 if not isinstance(obj, ignore_types):
                     # is this is a collection
                     iter(obj)
-                    col = ModelCollection(filter_list_by_qs(self._request.GET, obj))
+                    col = ModelCollection(filter_list(filters, obj))
                     col.__parent__ = self
 
                     if request:
